/**
 *   This file is part of AudioAnalyst.
 *
 *   AudioAnalyst is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   AudioAnalyst is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.lordikc.audioanalyst;

import java.util.Arrays;

import edu.emory.mathcs.jtransforms.fft.FloatFFT_1D;

/**
 * @author gilles
 * Set of audio processing functions.
 */
public class AudioProcessing {

	private static FloatFFT_1D fft=null;
	private static int fftLength;
	private static int[] hist=new int[Short.MAX_VALUE+1];

	/**
	 * Search the maximum absolute value of the given buffer.
	 * The computation removes 2% of the upper histogram
	 * @param buffer
	 * @return maximum value of the buffer
	 */
	public static short volume(short[] buffer) {
		Arrays.fill(hist, 0);
		for(short v : buffer) {
			int i=v;
			if(i<0) i=-i;
			if(i>Short.MAX_VALUE) i=Short.MAX_VALUE;
			hist[i]++;
		}
		int sum=0;
		int idx=Short.MAX_VALUE-1;
		while(sum<(buffer.length/50) && idx>0 ) {
			sum+=hist[idx--];
		}
		return (short)idx;
//		short vol=0;
//		for(short v : buffer) {
//			if(v>vol)
//				vol=(short)v;
//			else if (-v > vol)
//				vol=(short)-v;
//		}
//		return vol;
	}

	/**
	 * Compute the FFT of the input signal. The computation is performed in-place.
	 * FFT computation is performed through JTransforms and is multithreaded.
	 * @param fftRe: input wave signal, output spectrum
	 */
	public static void waveToSpectrum(float[] fftRe) {
		if(fft==null || fftRe.length!=fftLength) {
			fftLength=fftRe.length;
			fft=new FloatFFT_1D(fftLength);
		}
		fft.realForward(fftRe);
		float reN2=fftRe[1];
		for(int i=0;i<(fftLength/2);i++){
			float re=0.F;
			float im=0.F;
			re=fftRe[2*i];
			if(i>0)
				im=fftRe[2*i+1];
			float am=(float) Math.hypot(re,im);
			fftRe[i]=am;
		}
		fftRe[fftLength/2]=reN2;
		for(int i=(1+fftLength/2);i<fftLength;i++)
			fftRe[i]=0;
	}
}
