/**
 *   This file is part of AudioAnalyst.
 *
 *   AudioAnalyst is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   AudioAnalyst is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.lordikc.audioanalyst;

import java.awt.Color;

/**
 * @author Gilles Bernard
 * A set of configuration values.
 */
public class Config {

	private static double[] notesFreq=null;
	private static String[] notesName=null;
	private static double[] notesC = {
		32.70, 65.41,130.81, 	261.63, 	523.25, 	1046.50, 	2093.00, 	4186.01
	};
	private static double[] notesD = {
		36.71 , 73.42 ,146.83 ,293.66 ,587.33 ,1174.66 ,2349.32 ,4698.64
	};
	private static double[] notesE = {
		41.20, 82.41,164.81,329.63,659.26,1318.51,2637.02,5274.04
	};
	private static double[] notesF = {
		43.65, 87.31 ,174.61 ,349.23 ,698.46 ,1396.91 ,2793.83 ,5587.65
	};
	private static double[] notesG = {
		49.00, 98.00 ,196.00 ,392.00 ,783.99 ,1567.98 ,3135.96 ,6271.93 // Sol
	};
	private static double[] notesA = {
		55,110,220,440,880,1760,3520,7040 // La
	};
	private static double[] notesB = {
		61.74, 123.47 ,246.94 ,493.88 ,987.77 ,1975.53 ,3951.07 ,7902.13 // Si
	};

	private static float[] sampleRates = {8000,11025,22050,44100,48000};

	static {
		notesFreq = new double[notesA.length+notesB.length+notesC.length+notesD.length+notesE.length+notesF.length+notesG.length];
		notesName = new String[notesFreq.length];
		int idx=0;
		for(int i=0;i<notesA.length;i++) {
			notesFreq[idx]=notesC[i];
			notesName[idx]=new String("C"+i);
			idx++;
			notesFreq[idx]=notesD[i];
			notesName[idx]=new String("D"+i);
			idx++;
			notesFreq[idx]=notesE[i];
			notesName[idx]=new String("E"+i);
			idx++;
			notesFreq[idx]=notesF[i];
			notesName[idx]=new String("F"+i);
			idx++;
			notesFreq[idx]=notesG[i];
			notesName[idx]=new String("G"+i);
			idx++;
			notesFreq[idx]=notesA[i];
			notesName[idx]=new String("A"+i);
			idx++;
			notesFreq[idx]=notesB[i];
			notesName[idx]=new String("B"+i);
			idx++;
		}
	}

	private static double[] scalesFreq={
		0.1,2./30.,0.05,1./30.,0.02,1/75.,
		0.01,2./300.,0.005,1./300.,0.002,1/750.,
		0.001,2./3000., 0.0005,1./3000.,0.0002,1/7500.,
		0.0001,2./30000.,0.00005
	};
	private static double[] scalesBw={
		0.002,1/750.,
		0.001,2./3000., 0.0005,1./3000.,0.0002,1/7500.,
		0.0001,2./30000.,0.00005
	};
	private static double[] scalesSpectrogram={
		15,30,60,120,300,600
	};

	public static int sampleSizeInBits() {
		return 16;
	}

	public static float sampleRate2() {
		return 48000;
	}

	public static float defaultSampleRate() {
		return 48000;
	}

	public static float[] sampleRates() {
		return sampleRates;
	}

	public static int sampleSizeInBytes() {
		return sampleSizeInBits()/8;
	}

	/**
	 * Time scales for frequencies
	 * @return
	 */
	public static double[] timeScales() {
		return scalesFreq;
	}

	/**
	 * Time scales for bandwidth
	 * @return
	 */
	public static double[] timeScalesBw() {
		return scalesBw;
	}

	/**
	 * Default time scale for Oscilloscope
	 * @return
	 */
	public static int defaultTimeScale() {
		return 4;
	}

	/**
	 * Nb of vertical grids for Oscilloscope
	 * @return
	 */
	public static int nbGridsY() {
		return 6;
	}
	/**
	 * Nb of horizontal grids
	 * @return
	 */
	public static int nbGridsX() {
		return 10;
	}

	/**
	 * high level for volume
	 * @return
	 */
	public static float levelHigh() {
		return 0.933F;
	}

	/**
	 * medium level for volume
	 * @return
	 */
	public static float levelMedium() {
		return 0.733F;
	}

	/**
	 * Nb of bar in the display of the volume
	 * @return
	 */
	public static int nbVolumeBars() {
		return 15;
	}

	/**
	 * Default bandwidth for spectrum and spectrogram
	 * @return
	 */
	public static int defaultBandwidth() {
		return timeScalesBw().length-3;
	}

	/**
	 * Default central frequency for spectrum and spectrogram.
	 * @return
	 */
	public static int defaultCentralFrequency() {
		return 0;
	}

	/**
	 * Frequencies and names of notes.
	 * @return
	 */
	public static double[] getNotesFrequencies() { return notesFreq;}
	public static String[] getNotesNames() { return notesName;}

	/**
	 * Min samples per second to be read from the sound card.
	 * @return
	 */
	public static int minSamples() {
		return 8; // = 2^8
	}
	/**
	 * Max samples per second to be read from the sound card.
	 * @return
	 */
	public static int maxSamples() {
		return 16; // = 2^16
	}
	/**
	 * Default samples per second to be read from the sound card.
	 * @return
	 */
	public static int defaultSamples() {
		return 13; // = 2^13
	}

	public static final String HOLD = "mem";
	public static final String DECIMAL_MUSIC_SCALE = "dec";
	public static final Object ABOUT = "about";
	public static final String UPDOWNFRONT = "updown";
	public static final String DB = "db";
	public static final double MIN_DB = -20;
	public static final String NOISE = "noise";

	public static int defaultScaleSpectrogram() {
		return 1;
	}

	/**
	 * Scales to be used for the spectrogram. A scale is the
	 * total duration of the screen.
	 * @return
	 */
	public static double[] scalesSpectrogram() {
		return scalesSpectrogram;
	}

	/**
	 * The length of the spectrogram buffer the total size
	 * being length * nbSamples
	 * @return
	 */
	public static int spectrogramBufferLength() {
		return 1<<14;
	}

	private static Color[] color;

	public static Color[] getColors() { return color;}

	static {
		color = new Color[128];
		int i=0;
		int s1=64;
		int s2=96;
		for(i=0;i<s1;i++) { // Red
			color[i] = new Color((128*i)/s1,0,0);
		}
		for(;i<s2;i++) { // Yellow
			int v=(255*(i-s1))/(s2-s1);
			color[i] = new Color(128+v/2,v,0);
		}
		for(;i<128;i++) { // White
			int v=(252*(i-s2))/(128-s2);
			color[i] = new Color(255,255,v);
		}
			
	}

	public static int tileSize() {
		return 16;
	}

	public static long timeMaxNoise() {
		return 15;
	}
	public static long timeMinNoise() {
		return 1;
	}

	public static int globalBufferSize() {
		return 48000*1800;
	}
}

