/**
 *   This file is part of AudioAnalyst.
 *
 *   AudioAnalyst is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   AudioAnalyst is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.lordikc.audioanalyst;



/**
 * @author Gilles Bernard
 * Processing thread for Enveloppe.
 */
public class EnveloppeProcessThread extends Thread implements ISampleListener {
	private Enveloppe frame;
	private short[] sample;

	public EnveloppeProcessThread(int nbSamples, Enveloppe frame) {
		this.frame=frame;
		this.sample=new short[nbSamples];
		this.setName("EnveloppeProcessThread");
	}

	public void run() {
		while (!isInterrupted()) {
			try {
				synchronized(this) {
					wait();
					process_sample(sample);
				}
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	private void process_sample(short[] buffer) {
		short volume=AudioProcessing.volume(buffer);
		// Set the byte buffer
		frame.setVolume(volume); 
	}

	// Called by the AudioThread
	@Override
	public void process(short[] buffer) {
		synchronized(this) {
			sample=buffer;
			notify();
		}
	}

}
