/**
 *   This file is part of AudioAnalyst.
 *
 *   AudioAnalyst is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   AudioAnalyst is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.lordikc.audioanalyst;



/**
 * @author Gilles Bernard
 * Processing thread for spectrogram.
 */
public class SpectrogramProcessThread extends Thread implements ISampleListener {
	private Spectrogram frame;
	private short[] sample;
	private float[] fftRe;
	private float max;

	public SpectrogramProcessThread(int nbSamples, Spectrogram frame) {
		this.frame=frame;
		this.sample=new short[nbSamples];
		this.setName("SpectrogramProcessThread");
	}

	public void run() {
		while (!isInterrupted()) {
			try {
				synchronized(this) {
					wait();
					process_sample(sample);
				}
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	private void process_sample(short[] buffer) {
		compute_fft(buffer);
		// Set the raw short buffer
		frame.setVolume(AudioProcessing.volume(buffer));
		// Set the byte buffer
		frame.setBuffer(fftRe,max); 
	}

	private void compute_fft(short[] buffer) {
		short maxShort=0;
		for(short val:buffer) {
			if(Math.abs(val)>maxShort) maxShort=val;
		}
		// For FFT
		if (fftRe==null || fftRe.length!=buffer.length){
			fftRe=new float[buffer.length];
		}
		// float buffer range from -1 to 1
		for(int i=0;i<buffer.length;i++){
			fftRe[i]=(float)buffer[i]/(float)maxShort;
		}

		AudioProcessing.waveToSpectrum(fftRe);

		max=-Float.MAX_VALUE;
		for(int i=0;i<fftRe.length;i++){
			if(i<=1) fftRe[i]=0; // Remove DC and very low freq
			if(fftRe[i]>max) max=fftRe[i];
		}

	}

	// Called by the AudioThread
	@Override
	public void process(short[] buffer) {
		synchronized(this) {
			sample=buffer;
			notify();
		}
	}

}
