/**
 *   This file is part of AudioAnalyst.
 *
 *   AudioAnalyst is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   AudioAnalyst is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.lordikc.audioanalyst;



/**
 * @author Gilles Bernard
 * Processing thread for the spectrum application. It receives samples from the default audio line
 * and compute a FFT on it. It forwards the FFT to the main frame.
 * TODO: Use another mean to send the FFT to the Frame.
 */
public class SpectrumProcessThread extends Thread implements ISampleListener {
	private Spectrum frame;
	private short[] sample;
	private float[] fftRe;

	public SpectrumProcessThread(int nbSamples, Spectrum frame) {
		this.frame=frame;
		this.sample=new short[nbSamples];
	}

	public void run() {
		while (!isInterrupted()) {
			try {
				synchronized(this) {
					// Wait for available sample
					wait();
					process_sample(sample);
				}
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
	private void process_sample(short[] buffer) {
		short maxShort=Short.MIN_VALUE;
		for(short val:buffer) {
			if(Math.abs(val)>maxShort) maxShort=val;
		}
		// For FFT
		if (fftRe==null||fftRe.length!=buffer.length){
			fftRe=new float[buffer.length];
		}
		for(int i=0;i<buffer.length;i++){
			fftRe[i]=(float)buffer[i]/(float)maxShort;
		}

		AudioProcessing.waveToSpectrum(fftRe);

		double max=-Double.MAX_VALUE;
		for(int i=0;i<fftRe.length;i++){
			if(i<=1) fftRe[i]=0; // Remove DC and very low freq
			if(Math.abs(fftRe[i])>max) max=Math.abs(fftRe[i]);
		}

		frame.setVolume(AudioProcessing.volume(buffer));
		frame.setBuffer(fftRe,max); 
	}

	@Override
	public void process(short[] buffer) {
		synchronized(this) {
			sample=buffer;
			// Notify that a new sample is available
			notify();
		}
	}

}
