/**
 *   This file is part of AudioAnalyst.
 *
 *   AudioAnalyst is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   AudioAnalyst is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.lordikc.audioanalyst;

import java.awt.Color;
import java.text.DecimalFormat;

import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;

/**
 * @author Gilles Bernard
 * Set of low level utility functions
 */
public class Util {

	private static DecimalFormat df = new DecimalFormat ( ) ;
	static {
		df.setMaximumFractionDigits ( 2 ) ;
		df.setMinimumFractionDigits ( 0 ) ; 
	}

	/**
	 * Convert a frequence to a string for display purpose.
	 * @param d
	 * @return A human readable frequence
	 */
	public static String frequenceToText(double d) {
		if(d>=1000000000)
			return df.format(d/1000000000)+"G";
		if(d>=1000000)
			return df.format(d/1000000)+"M";
		if(d>=1000)
			return df.format(d/1000)+"K";
		return df.format(d)+"";
	}
	
	/**
	 * Convert a duration into a human readable string.
	 * @param d
	 * @return A human readable period of time
	 */
	public static String periodToText(double d) {
		if(d>=1)
			return df.format(d)+"s";
		if((d*1000)>=1)
			return df.format(d*1000)+"ms";
		if((d*1000000)>=1)
			return df.format(d*1000000)+"us";
		if((d*1000000000)>=1)
			return df.format(d*1000000000)+"ns";
		return df.format(d)+"s";
	}
	
	/**
	 * Create and display the about dialog.
	 * @param frame
	 */
	public static void about(JFrame frame) {
		JDialog dialog = new JDialog(frame);
		dialog.setTitle("About");
		JEditorPane jep = new JEditorPane("text/html",
			    "<b>"+frame.getClass().getSimpleName()+"</b> 0.3 by Lordikc. <br/>See <a href='http://www.lordikc.com'>www.lordikc.com</a> for more details.");
		dialog.add(jep);
		dialog.pack();
		dialog.setVisible(true);
		
	}

	/**
	 * Return the Color associated with the given FFT value.
	 * @param i
	 * @return a Color
	 */
	public static Color valueToColor(int i) {
		int idx=(i*Config.getColors().length)/Byte.MAX_VALUE;
		if(idx<0) idx=0;
		if(idx>=Config.getColors().length) idx=Config.getColors().length-1;
		return Config.getColors()[idx];
	}
	
	/**
	 * A monotonous clock value
	 * @return
	 */
	public static long clock() {
		//return System.currentTimeMillis();
		return System.nanoTime();
	}
	
	/**
	 * Scale for converting the clock value to second
	 * @return
	 */
	public static final long clock_to_second() {
		return 1000000000;
	}

	/**
	 * Compute the next power of 2 greated than val
	 * @param val
	 * @return
	 */
	public static int next_power_of_2(long val) {
		long p2=1;
		// Find next power of 2
		while (p2<val) p2*=2;
		return (int)p2;
	}

	/**
	 * @param volumeBuf
	 * @return
	 */
	public static double mean(double[] volumeBuf) {
		double sum=0;
		for(double val:volumeBuf) sum+=val;
		return sum/volumeBuf.length;
	}
}
