/**
 *   This file is part of AudioAnalyst.
 *
 *   AudioAnalyst is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   AudioAnalyst is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.lordikc.audioanalyst;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JPanel;

/**
 * @author Gilles Bernard
 * This class overrides JPanel in order to display the volume as a bar chart.
 */
public class VolumePanel extends JPanel {

	private static final long serialVersionUID = -8490458804473593155L;

	short volume=0;

	public VolumePanel() {
		setPreferredSize(new Dimension(10,150));
	}

	public void paint(Graphics g){
		super.paint(g);

		int step=getHeight()/Config.nbVolumeBars();
		int height=step-1;
		int heightMax=step*Config.nbVolumeBars();

		if(volume>0) {
			// att ranges from Config.MIN_DB to 0
			double att=10*Math.log10((double)volume/(double)Short.MAX_VALUE);
			if(att<Config.MIN_DB) att=Config.MIN_DB; 
			//volume=(short) (((2+Math.log10((double)volume/(double)Short.MAX_VALUE))/2)*heightMax);
			volume=(short) ((att-Config.MIN_DB)*heightMax/-Config.MIN_DB);
		}
		if(volume<0)
			volume=0;

		for(int i=0;i<Config.nbVolumeBars();i++) {
			int h1=getHeight()-(i+1)*step;
			if(i*step>=heightMax*Config.levelHigh())
				g.setColor(Color.RED);
			else if(i*step>=heightMax*Config.levelMedium())
				g.setColor(Color.YELLOW);
			else
				g.setColor(Color.GREEN);

			if(((i+0.5)*step)>volume)
				g.setColor(Color.GRAY);  

			g.fillRect(0, h1, getWidth(), height);
		}

	}
	/**
	 * @param buffer2
	 * @param min
	 * @param max
	 * @param index
	 * @param max2
	 * @param min2
	 * @param s
	 */
	public void setBuffer2(short[] buffer2) {
		volume=AudioProcessing.volume(buffer2);
	}

	public void setVolume(short vol) {
		volume=vol;
	}


}
